@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) Jarkko Hietaniemi, 1998,1999,2000. All Rights Reserved.
+Copyright (C) Jarkko Hietaniemi, 1998,1999,2000,2001. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of either:
@@ -403,10 +403,22 @@ apse_size_t apse_get_text_final_position(apse_t *ap) {
     return ap->text_final_position;
 }
 
+apse_bool_t apse_set_text_position_range(apse_t *ap,
+					 apse_size_t text_position_range) {
+    ap->text_position_range = text_position_range;
+
+    return 1;
+}
+
+apse_size_t apse_get_text_position_range(apse_t *ap) {
+    return ap->text_position_range;
+}
+
 void apse_reset(apse_t *ap) {
     _apse_reset_state(ap);
 
     ap->text_position = ap->text_initial_position;
+    ap->text_position_range = APSE_MATCH_BAD;
 
     ap->match_state = APSE_MATCH_STATE_BOT;
     ap->match_begin = APSE_MATCH_BAD;
@@ -455,6 +467,17 @@ apse_bool_t apse_set_edit_distance(apse_t *ap, apse_size_t edit_distance) {
     ap->match_end_bitvector =
 	(ap->pattern_size - 1) / APSE_BITS_IN_BITVEC;
 
+#ifdef APSE_DEBUGGING
+    if (ap->has_different_distances) {
+	printf("(edit distances: ");
+	printf("insertions = %ld, deletions = %ld, substitutions = %ld)\n",
+	       ap->edit_insertions,
+	       ap->edit_deletions,
+	       ap->edit_substitutions);
+    } else
+	printf("(edit_distance = %ld)\n", ap->edit_distance);
+#endif
+
 out:
     return ap->state && ap->prev_state;
 }
@@ -630,6 +653,7 @@ apse_t *apse_create(unsigned char*	pattern,
     ap->text_position		= 0;
     ap->text_initial_position	= 0;
     ap->text_final_position	= APSE_MATCH_BAD;
+    ap->text_position_range	= APSE_MATCH_BAD;
 
     ap->state			= 0;
     ap->prev_state		= 0;
@@ -684,14 +708,7 @@ apse_t *apse_create(unsigned char*	pattern,
 	printf("(single bitvector");
     else
 	printf("(multiple bitvectors");
-    if (ap->has_different_distances) {
-	printf(")\n(edit distances: ");
-	printf("insertions = %ld, deletions = %ld, substitutions = %ld)\n",
-	       ap->edit_insertions,
-	       ap->edit_deletions,
-	       ap->edit_substitutions);
-    } else
-	printf(", edit_distance = %ld)\n", ap->edit_distance);
+    printf(")\n");
 #endif
     okay = 1;
 
@@ -879,8 +896,16 @@ static apse_bool_t _apse_match_next_state(apse_t *ap) {
 		    active++;
 	    }
 	    if (equal == ap->edit_distance + 1 && ap->is_greedy == 0 ||
-		equal == 0 && ap->prev_active && active > ap->prev_active)
+		equal == 0 && ap->prev_active && active > ap->prev_active &&
+		!APSE_BIT_TST(ap->state,
+			      ap->edit_distance,
+			      ap->bitvectors_in_state,
+			      ap->text_position - ap->match_begin)) {
 		ap->match_begin = ap->text_position;
+#ifdef APSE_DEBUGGING
+		printf("(slide begin %d)\n", ap->match_begin);
+#endif
+	    }
 	    else if (active == 0)
 		_apse_match_fail(ap);
 	    ap->prev_active = active;
@@ -913,7 +938,7 @@ static apse_bool_t _apse_match_next_state(apse_t *ap) {
 	ap->match_state = APSE_MATCH_STATE_END;
 	APSE_DEBUG(printf("(match state %s)\n", apse_match_state_name(ap)));
 	ap->match_end	= ap->text_position - 1;
-	APSE_DEBUG(printf("(match end %ld)\n", ap->match_end));
+	APSE_DEBUG(printf("(match end %ld)\n",  ap->match_end));
     }
 
     return ap->match_state;
@@ -1210,7 +1235,8 @@ static apse_bool_t __apse_match(apse_t		*ap,
 	goto out;
     }
 
-    if (ap->pattern_size - ap->edit_deletions > ap->text_size - ap->text_initial_position) {
+    if (ap->pattern_size - ap->edit_deletions >
+	ap->text_size - ap->text_initial_position) {
 	ap->match_state   = APSE_MATCH_STATE_EOT;
 	ap->text_position = ap->text_size;
 	goto out;
@@ -1226,6 +1252,13 @@ static apse_bool_t __apse_match(apse_t		*ap,
 	_apse_reset_state(ap);
     }
 
+    if (ap->text_position_range != APSE_MATCH_BAD &&
+	ap->text_position - ap->text_initial_position >
+	ap->text_position_range) {
+	ap->match_state   = APSE_MATCH_STATE_END;
+	goto eot;
+    }
+
     ap->match_state = APSE_MATCH_STATE_SEARCH;
     APSE_DEBUG(printf("(match state %s)\n", apse_match_state_name(ap)));
     APSE_DEBUG(printf("(match search %ld)\n", ap->text_position));
@@ -1277,38 +1310,43 @@ static apse_bool_t _apse_match(apse_t 		*ap,
 			       unsigned char	*text,
 			       apse_size_t	text_size) {
     if (ap->use_minimal_distance) {
-	apse_size_t minimal_edit_distance;
-	apse_size_t previous_edit_distance;
-	apse_size_t next_edit_distance;
-	apse_size_t text_size_twice_plus_one = 2 * text_size + 1;
-
-	for (next_edit_distance = 0; ;
-	     next_edit_distance <= text_size_twice_plus_one) {
-	    apse_set_edit_distance(ap, next_edit_distance);
-	    if (__apse_match(ap, text, text_size))
-		break;
-	    previous_edit_distance = next_edit_distance;
-	    next_edit_distance = 2 * next_edit_distance + 1;
-	} 
-	minimal_edit_distance = 0;
-	if (next_edit_distance) {
-	    do {
-		minimal_edit_distance =
-		    (previous_edit_distance + next_edit_distance) / 2;
-		if (minimal_edit_distance == previous_edit_distance)
-		    break;
+	apse_set_edit_distance(ap, 0);
+	if (__apse_match(ap, text, text_size))
+	    return 1;
+	else {
+	    apse_size_t minimal_edit_distance;
+	    apse_size_t previous_edit_distance = 0;
+	    apse_size_t next_edit_distance;
+	    
+	    for (next_edit_distance = 1;
+		 next_edit_distance <= ap->pattern_size;
+		 next_edit_distance *= 2) {
 		apse_set_edit_distance(ap, next_edit_distance);
 		if (__apse_match(ap, text, text_size))
-		    next_edit_distance     = minimal_edit_distance;
-		else
-		    previous_edit_distance = minimal_edit_distance;
-	    } while (previous_edit_distance < next_edit_distance);
-	    minimal_edit_distance++;
+		    break;
+		previous_edit_distance = next_edit_distance;
+	    } 
+	    minimal_edit_distance = next_edit_distance;
+	    if (next_edit_distance > 1) {
+		do {
+		    minimal_edit_distance =
+			(previous_edit_distance + next_edit_distance) / 2;
+		    if (minimal_edit_distance == previous_edit_distance)
+			break;
+		    apse_set_edit_distance(ap, minimal_edit_distance);
+		    if (__apse_match(ap, text, text_size))
+			next_edit_distance     = minimal_edit_distance;
+		    else
+			previous_edit_distance = minimal_edit_distance;
+		} while (previous_edit_distance <= next_edit_distance);
+		if (!__apse_match(ap, text, text_size))
+		    minimal_edit_distance++;
+	    }
+	    apse_set_edit_distance(ap, minimal_edit_distance);
+	    __apse_match(ap, text, text_size);
+	    
+	    return 1;
 	}
-	apse_set_edit_distance(ap, minimal_edit_distance);
-	__apse_match(ap, text, text_size);
-
-	return 1;
     } else
 	return __apse_match(ap, text, text_size);
 }
@@ -1336,7 +1374,6 @@ apse_bool_t apse_match_next(apse_t *ap,
 apse_ssize_t apse_index(apse_t *ap,
 			 unsigned char *text, apse_size_t text_size) {
     apse_size_t did_match = _apse_match(ap, text, text_size);
-
     _apse_match_eot(ap);
     ap->match_state = APSE_MATCH_STATE_BOT;
 
